import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import Button from '../button/FoundationButton';
import Textbox from '../textbox/FoundationTextbox';
import TextboxSymbolText from '../textbox/FoundationTextboxSymbolText';

import Autocomplete from './Autocomplete';
import autocompleteState from './redux/autocompleteState';


const TestAutocomplete = Autocomplete({
    block: purebem.of('test-autocomplete'),
    displayName: 'TestAutocomplete',
    Button: Button,
    Textbox: Textbox,
    TextboxSymbolText: TextboxSymbolText,
});

const getOptions = () => {
    return [
        { value: 'A' },
        { value: 'B' },
        { value: 'C' },
    ];
};

test('should fetch data on focus', () => {
    const onDataFetch = jest.fn();
    const autocomplete = mount(
        <TestAutocomplete
            { ...autocompleteState() }
            onDataFetch={ onDataFetch } />
    );

    autocomplete.simulate('focus');

    expect(onDataFetch).toHaveBeenCalled();
});

test('should collapse expanded list when using escape', () => {
    const onCollapse = jest.fn();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            onCollapse={ onCollapse }
            options={ getOptions() } />
    );

    autocomplete.simulate('keydown', { which: 27 });

    expect(onCollapse).toHaveBeenCalled();
});

test('should collapse expanded list when clicking outside', () => {
    const onCollapse = jest.fn();

    mount(
        <TestAutocomplete
            expanded={ true }
            onCollapse={ onCollapse }
            options={ getOptions() } />
    );

    document.dispatchEvent(new Event('mousedown'));
    expect(onCollapse).toHaveBeenCalled();
});

test('should not select if expanded but no item has been selected', () => {
    const onSelect = jest.fn();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ -1 }
            onSelect={ onSelect }
            options={ getOptions() } />
    );

    autocomplete.simulate('keydown', { which: 13 });

    expect(onSelect).not.toHaveBeenCalled();
});

test('should trigger button click if expanded but no items', () => {
    const onSymbolTextClick = jest.fn();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ -1 }
            onSymbolTextClick={ onSymbolTextClick }
            options={ getOptions() }
            value={ 'yada' } />
    );

    autocomplete.simulate('keydown', { which: 13 });

    expect(onSymbolTextClick).toHaveBeenCalledWith('yada');
});

test('should select if expanded and item has been selected', () => {
    const onSelect = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ 0 }
            onSelect={ onSelect }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 13 });

    expect(onSelect).toHaveBeenCalledWith(options[0]);
});

test('should not select if expanded but fetching data', () => {
    const onSelect = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            fetching={ true }
            highlightedIndex={ 0 }
            onSelect={ onSelect }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 13 });

    expect(onSelect).not.toHaveBeenCalled();
});

test('should not change highlighted index when using up arrow and collapsed', () => {
    const onHighlightedIndexChange = jest.fn();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ false }
            onHighlightedIndexChange={ onHighlightedIndexChange } />
    );

    autocomplete.simulate('keydown', { which: 38 });

    expect(onHighlightedIndexChange).not.toHaveBeenCalled();
});

test('should not change highlighted index when using up arrow and fetching', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            fetching={ true }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 38 });

    expect(onHighlightedIndexChange).not.toHaveBeenCalled();
});

test('should not change highlighted index when using down arrow and collapsed', () => {
    const onHighlightedIndexChange = jest.fn();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ false }
            onHighlightedIndexChange={ onHighlightedIndexChange } />
    );

    autocomplete.simulate('keydown', { which: 40 });

    expect(onHighlightedIndexChange).not.toHaveBeenCalled();
});

test('should not change highlighted index when using down arrow and fetching', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            fetching={ true }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 40 });

    expect(onHighlightedIndexChange).not.toHaveBeenCalled();
});

test('should highlight last if no initial value when expanded and using up arrow', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ -1 }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 38 });

    expect(onHighlightedIndexChange).toHaveBeenCalledWith(options.length - 1);
});


test('should change highlighted index when expanded and using up arrow', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ 1 }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 38 });

    expect(onHighlightedIndexChange).toHaveBeenCalledWith(0);
});

test('should wrap around highlighted index when expanded and using up arrow on first', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ 0 }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 38 });

    expect(onHighlightedIndexChange).toHaveBeenCalledWith(options.length - 1);
});

test('should highlight first if no initial value when expanded and using down arrow', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ -1 }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 40 });

    expect(onHighlightedIndexChange).toHaveBeenCalledWith(0);
});

test('should change highlighted index when expanded and using down arrow', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ 1 }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 40 });

    expect(onHighlightedIndexChange).toHaveBeenCalledWith(2);
});

test('should wrap around highlighted index when expanded and using up arrow on first', () => {
    const onHighlightedIndexChange = jest.fn();
    const options = getOptions();
    const autocomplete = mount(
        <TestAutocomplete
            expanded={ true }
            highlightedIndex={ options.length - 1 }
            onHighlightedIndexChange={ onHighlightedIndexChange }
            options={ options } />
    );

    autocomplete.simulate('keydown', { which: 40 });

    expect(onHighlightedIndexChange).toHaveBeenCalledWith(0);
});
