import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import onClickOutside from 'react-onclickoutside';
import invariant from 'invariant';
import debounce from 'lodash.debounce';
import throttle from 'lodash.throttle';
import purebem from 'purebem';

import * as lang from '../../utils/lang';
import FoundationTether from '../misc/FoundationTether';
import isNullOrEmpty from '../../utils/isNullOrEmpty';
import modulus from '../../utils/modulus';
import reactInterpolate from '../../utils/reactInterpolate';


export default ({
    block,
    displayName,
    Button,
    Textbox,
    TextboxSymbolText,
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');
    invariant(Button !== undefined, 'Missing Button component.');
    invariant(Textbox !== undefined, 'Missing Textbox component.');
    invariant(TextboxSymbolText !== undefined, 'Missing TextboxSymbolText component.');

    return onClickOutside(React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            disabled: PropTypes.bool,
            expanded: PropTypes.bool,
            fetching: PropTypes.bool,
            highlightedIndex: PropTypes.number,
            onChange: PropTypes.func,
            onCollapse: PropTypes.func,
            onDataFetch: PropTypes.func,
            onHighlightedIndexChange: PropTypes.func,
            onSelect: PropTypes.func,
            onSymbolTextClick: PropTypes.func,
            optionMapper: PropTypes.func,
            options: PropTypes.arrayOf(
                PropTypes.shape({
                    content: PropTypes.node,
                    value: PropTypes.node.isRequired,
                })
            ),
            optionsTotal: PropTypes.number,
            placeholder: PropTypes.string,
            symbolMobileIconFallback: PropTypes.string,
            symbolText: PropTypes.string,
            tetherTo: PropTypes.string,
            value: PropTypes.string,
        },

        getDefaultProps() {
            return {
                disabled: false,
                expanded: false,
                fetching: false,
                highlightedIndex: -1,
                onChange: (/* { rawValue, value } */) => {},
                onCollapse: () => {},
                onDataFetch: (/* rawValue */) => {},
                onHighlightedIndexChange: (/* index */) => {},
                onSelect: (/* option */) => {},
                onSymbolTextClick: (/* value */) => {},
                optionMapper: (option) => option,
                options: [],
                optionsTotal: 0,
                placeholder: null,
                symbolMobileIconFallback: null,
                symbolText: '',
                tetherTo: 'content',
                value: '',
            };
        },

        getInitialState() {
            return {
                width: null,
            };
        },

        componentWillMount() {
            this.focusedElement = 'textbox';
            this.suppressDataFetch = false;
        },

        componentDidMount() {
            this.onDebouncedDataFetch = debounce(this.onDataFetch, 200);
            this.onThrottledResize = throttle(this.onResize, 100);

            window.addEventListener('resize', this.onThrottledResize);

            this.setWidth();
        },

        componentDidUpdate() {
            this.setWidth();
        },

        componentWillUnmount() {
            window.removeEventListener('resize', this.onThrottledResize);
        },

        handleClickOutside() {
            this.collapse();
        },

        onChange({ rawValue, value }) {
            this.props.onChange({ rawValue, value });
            this.onDebouncedDataFetch(rawValue);
        },

        onDataFetch(rawValue) {
            this.fetch(rawValue);
        },

        onFocus() {
            this.fetch(this.props.value);
            this.focus();
        },

        onKeyDown(evt) {
            const { expanded, fetching, highlightedIndex, onHighlightedIndexChange, onSymbolTextClick, options, value } = this.props;
            switch (evt.which) {
                case 9: // tab
                    if (expanded || fetching) {
                        // don't tab to next form control when expanded (or fetching).
                        evt.stopPropagation();
                        evt.preventDefault();

                        this.focusedElement = this.focusedElement === 'textbox'
                            ? 'button'
                            : 'textbox';

                        this.suppressDataFetch = true;
                        this[this.focusedElement].focus();
                    } else {
                        this.suppressDataFetch = false;
                    }

                    break;

                case 13: // enter
                    // don't scroll on page or risk submitting the form on enter here.
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (fetching || !expanded) {
                        return;
                    }

                    if (options[highlightedIndex]) {
                        this.onSelect(options[highlightedIndex]);
                    } else {
                        onSymbolTextClick(value);
                    }

                    break;

                case 27: // escape
                    this.collapse();

                    this.suppressDataFetch = true;
                    this.focus();

                    break;

                case 38: // up arrow
                    if (fetching || !expanded) {
                        return;
                    }

                    onHighlightedIndexChange(
                        highlightedIndex > -1
                            ? modulus(highlightedIndex - 1, options.length)
                            : options.length - 1
                    );

                    break;

                case 40: // down arrow
                    if (fetching || !expanded) {
                        return;
                    }

                    if (expanded) {
                        onHighlightedIndexChange(
                            highlightedIndex > -1
                                ? modulus(highlightedIndex + 1, options.length)
                                : 0
                        );
                    }

                    break;

                default:
                    break;
            }
        },

        onResize() {
            this.setWidth();
        },

        onSelect(option) {
            this.collapse();
            this.props.onSelect(option);
        },

        onSymbolTextClick() {
            this.collapse();
            this.props.onSymbolTextClick(this.props.value);
        },

        collapse() {
            if (this.props.expanded) {
                this.props.onCollapse();
            }
        },

        fetch(value) {
            if (!this.suppressDataFetch) {
                this.props.onDataFetch(value);
            }

            this.suppressDataFetch = false;
        },

        focus() {
            this.focusedElement = 'textbox';
            this.textbox.focus();
        },

        setWidth() {
            const el = ReactDOM.findDOMNode(this);
            const width = el.offsetWidth;

            if (this.state.width !== width) {
                this.setState({ width });
            }
        },

        renderList() {
            const { expanded, fetching, optionMapper, options, optionsTotal, symbolText, tetherTo } = this.props;
            if (!expanded) {
                return null;
            }

            const matches = optionsTotal > 0;

            return (
                <FoundationTether
                    target={ () => ReactDOM.findDOMNode(this) }
                    tetherTo={ tetherTo }
                    width={ this.state.width !== null ? `${this.state.width}px` : null }>
                    <div className={ purebem.many(block('list'), 'ignore-react-onclickoutside') }>
                        <div className={ block('list-content', { fetching }) }>
                            { [].map.call(options.map(optionMapper), this.renderListItem) }
                            <div className={ block('list-content-summary', { matches }) }>
                                {
                                    matches
                                        ? reactInterpolate(lang.pgettext('foundation-autocomplete', 'Label--showing-results[%(showing)s,%(total)s]'), { showing: options.length, total: optionsTotal })
                                        : lang.pgettext('foundation-autocomplete', 'Label--no-matches')
                                }
                            </div>
                            <hr className={ block('list-content-separator') } />
                            <Button
                                onClick={ this.onSymbolTextClick }
                                onKeyDown={ this.onKeyDown }
                                ref={ (c) => this.button = c }
                                ui={ ['mobile-stretched'] }>
                                { symbolText }
                            </Button>
                        </div>
                    </div>
                </FoundationTether>
            );
        },

        renderListItem(option, index) {
            const { highlightedIndex } = this.props;
            const highlighted = highlightedIndex === index;
            const content = option.content || option.value;

            return (
                <div
                    className={ block('list-item', { highlighted }) }
                    key={ option.value }
                    onClick={ () => this.onSelect(option) }>
                    { content }
                </div>
            );
        },

        render() {
            const { disabled, fetching, placeholder, symbolMobileIconFallback, symbolText, value } = this.props;
            return (
                <div
                    className={ block() }
                    onFocus={ this.onFocus }
                    onKeyDown={ this.onKeyDown }>
                    <TextboxSymbolText
                        disabled={ disabled }
                        loading={ fetching }
                        mobileIconFallback={ symbolMobileIconFallback }
                        onClick={ this.onSymbolTextClick }
                        text={ symbolText }>
                        <Textbox
                            disabled={ disabled }
                            onChange={ this.onChange }
                            placeholder={ placeholder }
                            ref={ (c) => this.textbox = c }
                            ui="autocomplete"
                            value={ value } />
                    </TextboxSymbolText>
                    { this.renderList() }
                </div>
            );
        },

    }));

};
